library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity CONFIG is
	Port ( CLK      : in  std_logic; -- 18.432 MHz
			 CE       : in  std_logic; 
 			 RST      : in  std_logic; 
 			 CFG_DATA : in  std_logic_vector( 7 downto 0 );
			 CFG_RD   : in  std_logic;
			 READY    : in  std_logic;
			 DONE     : in  std_logic;
			 BUSY     : out std_logic;
			 CCLK     : out std_logic;
			 DIN      : out std_logic;
			 PROG     : out std_logic );
end CONFIG;

architecture RTL of CONFIG is

constant CCLK_MAX : std_logic_vector( 6 downto 0 ) := "1010000";
signal   CCLK_CNT : std_logic_vector( 6 downto 0 );

signal CFG_PARALLEL : std_logic_vector( 7 downto 0 );
signal BYTE_CNT     : std_logic_vector( 3 downto 0 );

signal   STATE   : std_logic_vector(3 downto 0);
constant S0_INT  : std_logic_vector(3 downto 0) := "0000"; -- 0
constant S1_INT  : std_logic_vector(3 downto 0) := "0001"; -- 1
constant S2_INT  : std_logic_vector(3 downto 0) := "0010"; -- 2
constant S3_INT  : std_logic_vector(3 downto 0) := "0011"; -- 3

constant S0_ENB  : std_logic_vector(3 downto 0) := "0100"; -- 4
constant S0_CFG  : std_logic_vector(3 downto 0) := "0101"; -- 5
constant S1_CFG  : std_logic_vector(3 downto 0) := "0110"; -- 6
constant S2_CFG  : std_logic_vector(3 downto 0) := "0111"; -- 7

constant S3_CFG  : std_logic_vector(3 downto 0) := "1000"; -- 8
constant S0_REM  : std_logic_vector(3 downto 0) := "1001"; -- 9
constant S1_REM  : std_logic_vector(3 downto 0) := "1010"; -- 10

begin

process( CLK, RST ) begin
	if( RST='1') then
			BUSY <= '0';
			CCLK <= '0';
			DIN  <= '0';
			PROG <= '1';
			CCLK_CNT <= (others=>'0'); 
			BYTE_CNT <= (others=>'0');
			STATE <= S0_INT;
	elsif( CLK'event and CLK='1') then
		case STATE is
	-------------------------------------------
	-------------------------------------------
			when S0_INT =>
				if( CE='1') then
					BUSY  <= '0';
					PROG  <= '1';
					STATE <= S1_INT;
				else
					STATE <= S0_INT;
				end if;
			when S1_INT =>
				if( CE='1') then
					PROG  <= '0';
					STATE <= S2_INT;
				else
					STATE <= S1_INT;
				end if;
			when S2_INT =>
				if( CE='1') then
					PROG  <= '1';
					STATE <= S3_INT;
 			   else
					STATE <= S2_INT;
				end if;
			when S3_INT =>
				if( CE='1') then
					STATE <= S0_ENB;
				else
					STATE <= S3_INT;
				end if;
	-------------------------------------------
	-------------------------------------------
			when S0_ENB =>
				if( CFG_RD = '1') then		
					STATE <= S0_CFG;
				else
					BUSY <= '0';
					CCLK <= '0';
					DIN  <= '0';
					PROG <= '1';	
					STATE <= S0_ENB;
				end if;
	-------------------------------------------
	-------------------------------------------				
			when S0_CFG =>
				if( DONE = '1') then
					BUSY  <= '0';
					STATE <= S0_REM;
				elsif( CFG_RD = '1') then
					CFG_PARALLEL <= CFG_DATA;
					BUSY  <= '1';
					STATE <= S1_CFG;
				else
					BUSY  <= '0';
					STATE <= S0_CFG;
				end if;
			when S1_CFG =>
				if( READY = '1') then
					STATE <= S2_CFG;
				else
					STATE <= S1_CFG;
				end if;
			when S2_CFG =>
				CCLK  <= '0';
		 		DIN   <= CFG_PARALLEL(7);
				STATE <= S3_CFG;
			when S3_CFG =>
				if( BYTE_CNT = "1000" ) then
					BYTE_CNT  <= (others=>'0');
					STATE <= S0_CFG;					
				else
 					BYTE_CNT <= BYTE_CNT + '1';
					CFG_PARALLEL <= CFG_PARALLEL( 6 downto 0 ) & '0';
					CCLK  <= '1';
					STATE <= S2_CFG;
				end if;
	-------------------------------------------
	-------------------------------------------			
			when S0_REM =>
				if( CCLK_CNT = CCLK_MAX ) then
					CCLK_CNT <= (others=>'0');
					CCLK <= '0';
					DIN  <= '0';
 				   PROG <= '1';
					STATE <= S0_ENB;
				else
					CCLK      <= '1';
					STATE <= S1_REM;
				end if;
			when S1_REM =>
					CCLK      <= '0';
					CCLK_CNT  <= CCLK_CNT + '1';
					STATE <= S0_REM;
	-------------------------------------------
	-------------------------------------------	
			when others => STATE <= S0_INT;
		end case;
	end if;
end process;

end RTL;


