library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity USB_READ is
	Port ( CLK      : in  std_logic;
			 RXF_N    : in  std_logic;
			 RD_N     : out std_logic;
			 USB_DATA : in  std_logic_vector( 7 downto 0 );
			 USB_BSY  : out std_logic;
			 TGT_RDY  : in  std_logic;
			 TGT_DO   : out std_logic_vector( 7 downto 0 );
			 TGT_RD   : out std_logic );
end USB_READ;

architecture RTL of USB_READ is

signal   RXF_REG  : std_logic;
signal   RD_STATE : std_logic_vector( 1 downto 0 );
constant S0       : std_logic_vector( 1 downto 0 ) := "00";
constant S1       : std_logic_vector( 1 downto 0 ) := "01";
constant S2       : std_logic_vector( 1 downto 0 ) := "10";
constant S3       : std_logic_vector( 1 downto 0 ) := "11";

begin

USB_BSY <= '0' when( RD_STATE = "00" ) else '1';

process( CLK ) begin
	if( CLK'event and CLK='1') then
		RXF_REG <= not RXF_N;
	end if;
end process;

--***                      *--
--**  READ STATE MACHINE  **--
--*                      ***--
process( CLK ) begin
	if( CLK'event and CLK='1') then
		case RD_STATE is
			when S0 =>
				if( RXF_REG='1') then
					RD_N     <= '0';
					RD_STATE <= S1;
				else						
					RD_STATE <= S0;
				end if;
			when S1 =>
				TGT_DO	<= USB_DATA;
				RD_STATE <= S2;
			when S2 =>
				if( TGT_RDY = '1') then
					RD_N     <= '1'; 
					TGT_RD   <= '1';
					RD_STATE <= S3;
				else
					RD_STATE <= S2;
				end if;
			when S3 =>
				TGT_RD   <= '0';
				RD_STATE <= S0;
			when others => RD_STATE <= S0;
		end case;
	end if;
end process;

end RTL;
